// AFL Ladder Widget 
// Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Tom Armitage
// http://expat.dyndns.org
// Version 2.0.2
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// Change Log:
//   2.0.5 - Fix for GWS Giants and 18 team competition
//   2.0.4 - Fix for Gold Coast Suns and 17 team competition
//   2.0.3 - Fix for incorrect URL in FinalSiren code.
//   2.0.2 - Fix for FinalSiren Chg column, now tries to autodetect pts column
//   2.0.1 - Changes for 2010 Season: added % column, minor maintainability updates
//   2.0.0 - rewrite &  new graphics for 2009 Season
//   1.5.6 - fixes for 10.5.2, all CSS and format change
//   1.5.5 - fixes for finalsiren changes, added refresh button, preferences changes
//   1.5.4 - fixes for season 2008
//   1.5.3 - fixes for changes to sportal.com.au
//   1.5.2 - fixes for ladder table changes after start of season 2007
//   1.5.1 - added selection of ladder data sources - afl.com.au, sportal.com.au, finalsiren.com
//   1.5.0 - fixes due to AFL website, now uses XML file and added tooltip data
//   1.4.3 - fixed problems introduced by 10.4.7 update
//   1.4.2 - fixed a problem with the last ladder position data
//   1.4.1 - fixes due to AFL website changes.
//   1.4  - fixed for 10.4.4, lots of rewriting...
//   1.3  - changed look to make more compact, added favourite team preference and diferent 
//          colours for top & bottom 8, lots of code tidying.
//   1.2  - fixed Sydney Swans formatting problem
//   1.1  - fixed updates and a few other potential bugs and tidied up code
//   1.0  - initial release
//   
//----------------------
// Global variables used by the widget

var ladderURL           = 0;
var complete            = false;
var version             = "2.0.5";
var processingRequest1  = false;
var processingRequest2  = false;
var homeURL             = "http://expat.dyndns.org/afl-ladder-widget";
var homeString          = "expat.dyndns.org/afl-ladder-widget";
var updateQueryURL      = "http://expat.dyndns.org/cgi-bin/versionQuery.cgi?AFLLadder";
// URL for testing update display
// var updateQueryURL      = "http://expat.dyndns.org/cgi-bin/versionQueryTest.cgi?AFLLadder";
var numberOfPastGames   = 6;
var favourite           = 0;
var updates             = true;
var debugAlerts         = false;
var debugData           = false;

var teamData = [
//  teamid  shortname            Website URL                     AFL.com                   finalsiren.com XML        finalsiren.com            Herald Sun                SMH                       Fox Sports                sportal.com.au            Yahoo 7 Sports
    ["40",  "Adelaide Crows",   "http://afc.com.au",             "Adelaide",               "Adelaide",               "Adelaide",               "Adelaide",               "Adelaide",               "Adelaide",               "Adelaide",               "Adelaide Crows"],
    ["36",  "Brisbane Lions",   "http://lions.com.au",           "Brisbane Lions",         "Brisbane Lions",         "Brisbane Lions",         "Brisbane",               "Brisbane",               "Brisbane",               "Brisbane",               "Brisbane Lions"], 
    ["43",  "Carlton",          "http://carltonfc.com.au",       "Carlton",                "Carlton",                "Carlton",                "Carlton",                "Carlton",                "Carlton",                "Carlton",                "Carlton"],
    ["38",  "Collingwood",      "http://collingwoodfc.com.au",   "Collingwood",            "Collingwood",            "Collingwood",            "Collingwood",            "Collingwood",            "Collingwood",            "Collingwood",            "Collingwood"],
    ["37",  "Essendon",         "http://bombersfc.com.au",       "Essendon",               "Essendon",               "Essendon",               "Essendon",               "Essendon",               "Essendon",               "Essendon",               "Essendon"],
    ["34",  "Fremantle",        "http://fremantlefc.com.au",     "Fremantle",              "Fremantle",              "Fremantle",              "Fremantle",              "Fremantle",              "Fremantle",              "Fremantle",              "Fremantle"],
    ["30",  "Geelong",          "http://gfc.com.au",             "Geelong Cats",           "Geelong",                "Geelong",                "Geelong",                "Geelong",                "Geelong",                "Geelong",                "Geelong Cats"],
    ["  ",  "Gold Coast Suns",  "http://goldcoastfc.com.au",     "Gold Coast",             "Gold Coast",             "Gold Coast",             "Gold Coast",             "Gold Coast",             "Gold Coast",             "Gold Coast",             "Gold Coast Suns"],
    ["  ",  "GWS Giants",       "http://www.gwsgiants.com.au",   "Greater Western Sydney", "GWS Giants",             "GWS Giants",             "Greater Western Sydney", "GWS",                    "Greater Western Sydney", "Greater Western Sydney", "GWS Giants"],  
    ["32",  "Hawthorn",         "http://hawthornfc.com.au",      "Hawthorn",               "Hawthorn",               "Hawthorn",               "Hawthorn",               "Hawthorn",               "Hawthorn",               "Hawthorn",               "Hawthorn"],
    ["45",  "North Melbourne",  "http://kangaroos.com.au",       "North Melbourne",        "North Melbourne",        "North Melbourne",        "North Melbourne",        "Kangaroos",              "North Melbourne",        "North Melbourne",        "North Melbourne"],
    ["31",  "Melbourne",        "http://melbournefc.com.au",     "Melbourne",              "Melbourne",              "Melbourne",              "Melbourne",              "Melbourne",              "Melbourne",              "Melbourne",              "Melbourne"],
    ["41",  "Port Adelaide",    "http://portadelaidefc.com.au",  "Port Adelaide",          "Port Adelaide",          "Port Adelaide",          "Port Adelaide",          "Port Adelaide",          "Port Adelaide",          "Port Adelaide",          "Port Adelaide"],
    ["42",  "Richmond",         "http://richmondfc.com.au",      "Richmond",               "Richmond",               "Richmond",               "Richmond",               "Richmond",               "Richmond",               "Richmond",               "Richmond"],
    ["39",  "St Kilda",         "http://saints.com.au",          "St Kilda",               "St Kilda",               "St Kilda",               "St Kilda",               "St Kilda",               "St Kilda",               "St Kilda",               "St Kilda"],
    ["33",  "Sydney Swans",     "http://sydneyswans.com.au",     "Sydney Swans",           "Sydney",                 "Sydney",                 "Sydney",                 "Sydney",                 "Sydney",                 "Sydney",                 "Sydney Swans"],
    ["35",  "West Coast",       "http://westcoasteagles.com.au", "West Coast Eagles",      "West Coast",             "West Coast",             "West Coast",             "West Coast",             "West Coast",             "West Coast",             "West Coast Eagles"],
    ["44",  "Western Bulldogs", "http://westernbulldogs.com.au", "Western Bulldogs",       "Western Bulldogs",       "Western Bulldogs",       "Western Bulldogs",       "Bulldogs",               "Bulldogs",               "Western Bulldogs",       "Western Bulldogs"],
];

var groundNames = [
//	 stadium name                short stadium name	
	["Melbourne Cricket Ground", "M.C.G."],
	["Sydney Cricket Ground",    "S.C.G."],
    ["Subiaco Oval",             "SUBIACO"]
];

// This table contains the source URLs which the widget uses to look up 
// the ladderdata. To add a new URL, first add it to the list of URL names
// then add it to the array in the next position.  Column [0] is used to 
// enable or disable the URL from being used, useful for tesing and for 
// removing URLs that aren't working right now but may work again in the 
// future

// Also add the team names as presented by the ladder at the new URL to 
// the teamData array in the corresponding position.

// URL Names
var aflURL = 0;
var finalsirenXML = 1;
var finalsirenURL = 2;
var heraldsunURL = 3;
var smhURL = 4;
var foxsportsURL = 5;
var sportalURL = 6;
var yahoo7URL = 7;

var sourceURLs = [
//  enabled, name,              URL
    [0,      aflURL,            "http://afl.com.au/ajax/ladder.aspx?teamid=0&seasonid=82&year=2011"],   // untested 2012
    [0,      finalsirenXML,     "http://finalsiren.com/feed/xml/ladder/"],                              // tested 2012 - bad
    [1,      finalsirenURL,     "http://finalsiren.com/AFLLadder.asp"],                                 // tested 2012 - ok
    [0,      heraldsunURL,      "http://superstats.heraldsun.com.au/ss2/ladders/ladder-lot-2012.html"], // untested 2012
    [1,      smhURL,            "http://livescores.smh.com.au/afl/ladder.html"],                        // tested 2012 - ok
    [1,      foxsportsURL,      "http://www.foxsports.com.au/afl/"],                                    // tested 2012 - ok
    [1,      sportalURL,        "http://www.sportal.com.au/afl-ladder/2012014"],                        // tested 2012 - ok
    [1,      yahoo7URL,         "http://au.sports.yahoo.com/afl/ladder/"],                              // tested 2012 - ok
];

//----------------------
// Array to store the downloaded ladder data
var ladderArrayColumns = 11;
var ladderArray = [
    ["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],       // first place
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
	["", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],       // last place
];

//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    dashcode.setupParts();

    // set up link string
    document.getElementById("linkString").innerHTML = "<a href='javascript:openInBrowser()'>" + homeString + "</a>";
    // set up other strings
    document.getElementById("versionString").innerText = "Version " + version;

    // initialise variables
    complete = false;
}

//
// Function: remove()
// Called when the widget has been removed from the Dashboard
//
function remove()
{
    // Stop any timers to prevent CPU usage
    // Remove any preferences as needed
    // widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
}

//
// Function: hide()
// Called when the widget has been hidden
//
function hide()
{
    // Stop any timers to prevent CPU usage
    
    // save last ladder
}

//
// Function: show()
// Called when the widget has been shown
//
function show()
{
    // Restart any timers that were stopped on hide
    
    // load favourite team
    loadPreferences();
    
    // initialise variables
    // set the initial ladder data URL to the first one
    ladderURL = 0;
    complete = false;
    
    // check that ladder address is enabled
    for (x = 0; x < sourceURLs.length; x++) {
        if ( sourceURLs[ladderURL][0] == 1) {
            break;  
        }
        else {
            ladderURL = ladderURL + 1;
        }
    }
    
    // fetch new ladder data
    getLadderData(ladderURL);
    
    // get update data
    getUpdateData();
    
    // update ladder data lists
    document.getElementById("teamList").object.reloadData();
    document.getElementById("playedList").object.reloadData();
}

//
// Function: loadPreferences()
//
function loadPreferences()
{
    // Retrieve any preference values that you need to be synchronized here
    // Use this for an instance key's value:
    // instancePreferenceValue = widget.preferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
    //
    // Or this for global key's value:
    // globalPreferenceValue = widget.preferenceForKey(null, "your-key");
    
    if (widget.preferenceForKey("favourite") != 0) {
        favourite = widget.preferenceForKey("favourite");
    }
    else {
        favourite = 0;
    }
    document.getElementById("teamPopup").object.setSelectedIndex(favourite);
    debugAlert("loadPreferences: favourite preference = " + favourite);
    
    if (widget.preferenceForKey("updates") != undefined) {
        updates = widget.preferenceForKey("updates");

    }
    else {
        updates = true;
    }
    // set correct state of checkbox button.
    document.getElementById("checkboxbutton").checked = updates;
    debugAlert("loadPreferences: update preference = " + updates);
}

//
// Function: sync
// called for .Mac stuff - not used here
//
function sync()
{
}

//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
    
    // set favourite
    // Values you provide
    favourite = document.getElementById("teamPopup").object.getSelectedIndex();
    debugAlert("showFront: favourite preference = " + favourite);

    // set the preference in file
	if(window.widget) {
	    widget.setPreferenceForKey(favourite, "favourite");
        debugAlert("showFront: wrote favourite preference = " + favourite);
        widget.setPreferenceForKey(updates, "updates");
        debugAlert("showFront: wrote updates preference = " + updates);        
    }
    // change favourite highlight
    highlightFavourite();
}

//-----------------------------------
// attach dashboard handlers

if (window.widget) {
    widget.onremove = remove;
    widget.onhide = hide;
    widget.onshow = show;
    widget.onsync = sync;
}


//-----------------------------------
// Refresh button handler
function refresh(event)
{
    // Initialise variables
    ladderURL = 0;
    complete = false;
    
    // fetch data
    getLadderData(ladderURL);
    getUpdateData();
}

//---------------------------------------------------------------------
// This function finds the inner most text of an element, used to 
// clean retrieved data
function findNodeValue(aNode) {
    // If it's text, just return it
    if (aNode.nodeValue != null)
        return aNode.nodeValue;

    // Otherwise, concatenate the contents of the children
    var childText = "";
    var nodeChild = aNode.firstChild;
    while (nodeChild) {
        childText += findNodeValue(nodeChild) + " ";
        nodeChild = nodeChild.nextSibling;
    }
    return childText;
}

//---------------------------------------------------------------------
// Return the text contents of a table cell, with tags and whitespace removed

function cleanCell(cell) {
    var pattern=/Kilda/gi;
    var pattern2=/ \(/gi;

    cell.normalize();
    // Try to find the innermost text, which may be surrounded by <b></b> or whatever
    var cellString = findNodeValue(cell);

    var cellText = new String(cellString);
    // Convert nbsp to space
    cellText = cellText.replace(/\xa0/g, " ");

    // Trim off information in brackets ( )
    
    if (cellText.match(pattern2)) {
        temp = cellText.split(" (");
        debugAlert("cleanCell: Trimmed brackets " + cellText + " -> " + temp[0]);
        cellText = temp[0];  
    }

    // Remove junk
    cellText = cellText.replace(/[^\w:,'\.\/ ]+/g, "");    //'

    // Remove leading junk numbers from some websites 
    // & handle St Kilda spelt as 'St. Kilda' 
    temp = "";
    temp = cellText.split(". ");
    
    if (cellText.match(pattern)) {
        if (temp.length == 2) {
            cellText=temp[0] + " " + temp[1];
        }
        else if (temp.length == 3) {
            cellText=temp[1] + " " + temp[2];
        }
    }
    else {
        if (temp.length == 2) {
            cellText=temp[1];
        }
        else if (temp.length == 3) {
            cellText=temp[1] + " " + temp[2];
        }
    }
  
    // Remove whitespace
    cellText = cellText.replace(/^\s*(.*?)\s*$/, "$1");

    if (cellText.length == 0) {
        cellText = null;
    }
    return cellText;
}

// This object implements the dataSource methods for the list.
var teamArrayLadder = {
	
	// Sample data for the content of the list. 
	// Your application may also fetch this data remotely via XMLHttpRequest.
	_rowData: [ "",  "", "", "", "", "", "", "", "", "", "", "", "", "", "", ""],
    
	// The List calls this method to find out how many rows should be in the list.
	numberOfRows: function() {
		return this._rowData.length;
	},
	
	// The List calls this method once for every row.
	prepareRow: function(rowElement, rowIndex, templateElements) {
		// templateElements contains references to all elements that have an id in the template row.
		// Ex: set the value of an element with id="label".
		if (templateElements.label) {
			templateElements.label.innerText = this._rowData[rowIndex];
		}

		// Assign a click event handler for the row.
		rowElement.onclick = function(event) {
			// Do something interesting
            widget.openURL(ladderArray[rowIndex][9]);
		};
	}
};

// This object implements the dataSource methods for the list.
var playedArrayLadder = {
	
	// Sample data for the content of the list. 
	// Your application may also fetch this data remotely via XMLHttpRequest.
	_rowData: [ "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "" ],
	
	// The List calls this method to find out how many rows should be in the list.
	numberOfRows: function() {
		return this._rowData.length;
	},
	
	// The List calls this method once for every row.
	prepareRow: function(rowElement, rowIndex, templateElements) {
		// templateElements contains references to all elements that have an id in the template row.
		// Ex: set the value of an element with id="label".
		if (templateElements.label2) {
			templateElements.label2.innerText = this._rowData[rowIndex];
		}
	}
};
// This object implements the dataSource methods for the list.
var wonArrayLadder = {
	
	// Sample data for the content of the list. 
	// Your application may also fetch this data remotely via XMLHttpRequest.
	_rowData: ["", "", "", "", "", "", "", "", "", "", "", "", "", "", "", ""],
	
	// The List calls this method to find out how many rows should be in the list.
	numberOfRows: function() {
		return this._rowData.length;
	},
	
	// The List calls this method once for every row.
	prepareRow: function(rowElement, rowIndex, templateElements) {
		// templateElements contains references to all elements that have an id in the template row.
		// Ex: set the value of an element with id="label".
		if (templateElements.label3) {
			templateElements.label3.innerText = this._rowData[rowIndex];
		}
	}
};

// This object implements the dataSource methods for the list.
var lostArrayLadder = {
	
	// Sample data for the content of the list. 
	// Your application may also fetch this data remotely via XMLHttpRequest.
	_rowData: ["", "", "", "", "", "", "", "", "", "", "", "", "", "", "", ""],
	
	// The List calls this method to find out how many rows should be in the list.
	numberOfRows: function() {
		return this._rowData.length;
	},
	
	// The List calls this method once for every row.
	prepareRow: function(rowElement, rowIndex, templateElements) {
		// templateElements contains references to all elements that have an id in the template row.
		// Ex: set the value of an element with id="label".
		if (templateElements.label4) {
			templateElements.label4.innerText = this._rowData[rowIndex];
		}
	}
};

// This object implements the dataSource methods for the list.
var drawnArrayLadder = {
	
	// Sample data for the content of the list. 
	// Your application may also fetch this data remotely via XMLHttpRequest.
	_rowData: ["", "", "", "", "", "", "", "", "", "", "", "", "", "", "", ""],
	
	// The List calls this method to find out how many rows should be in the list.
	numberOfRows: function() {
		return this._rowData.length;
	},
	
	// The List calls this method once for every row.
	prepareRow: function(rowElement, rowIndex, templateElements) {
		// templateElements contains references to all elements that have an id in the template row.
		// Ex: set the value of an element with id="label".
		if (templateElements.label5) {
			templateElements.label5.innerText = this._rowData[rowIndex];
		}
	}
};

// This object implements the dataSource methods for the list.
var pointsArrayLadder = {
	
	// Sample data for the content of the list. 
	// Your application may also fetch this data remotely via XMLHttpRequest.
	_rowData: ["", "", "", "", "", "", "", "", "", "", "", "", "", "", "", ""],
	
	// The List calls this method to find out how many rows should be in the list.
	numberOfRows: function() {
		return this._rowData.length;
	},
	
	// The List calls this method once for every row.
	prepareRow: function(rowElement, rowIndex, templateElements) {
		// templateElements contains references to all elements that have an id in the template row.
		// Ex: set the value of an element with id="label".
		if (templateElements.label6) {
			templateElements.label6.innerText = this._rowData[rowIndex];
		}
	}
};

// This object implements the dataSource methods for the list.
var percentArrayLadder = {
	
	// Sample data for the content of the list. 
	// Your application may also fetch this data remotely via XMLHttpRequest.
	_rowData: ["", "", "", "", "", "", "", "", "", "", "", "", "", "", "", ""],
	
	// The List calls this method to find out how many rows should be in the list.
	numberOfRows: function() {
		return this._rowData.length;
	},
	
	// The List calls this method once for every row.
	prepareRow: function(rowElement, rowIndex, templateElements) {
		// templateElements contains references to all elements that have an id in the template row.
		// Ex: set the value of an element with id="label".
		if (templateElements.label1) {
			templateElements.label1.innerText = this._rowData[rowIndex];
		}
	}
};

function getLadderData (url) {
if (processingRequest1 == false) {
        
        processingRequest1 = true;
        complete = false;
        // show the progress spinner
        document.getElementById("progress").style.visibility = "visible";
        document.getElementById("error").style.visibility = "hidden";
        
        debugAlert("onWidgetShow: sourceURL = " + sourceURLs[url][2]);
        // request ladder data
        xmlhttp = false;
        xmlhttp = new XMLHttpRequest();       
        xmlhttp.onreadystatechange =  processReqChange;
        if (xmlhttp.overrideMimeType) {
            xmlhttp.overrideMimeType("text/xml");
        }
        xmlhttp.open("GET", sourceURLs[url][2], true);
        xmlhttp.setRequestHeader('If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT');
        xmlhttp.send(null);
        ladderLoaded = false;
	}
}

function getUpdateData () {
    if (document.getElementById("checkboxbutton").checked == true) {
	if (processingRequest2 == false){
   		processingRequest2 = true;
		// request version data
		xmlhttp2 = false;
		xmlhttp2 = new XMLHttpRequest();
		xmlhttp2.onreadystatechange =  processReqChangeUpdate;
		xmlhttp2.open("GET", updateQueryURL, true);
		xmlhttp2.setRequestHeader('If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT');
		xmlhttp2.send(null);
	}
    }
}

//----------------------
// This function handles the fetching of the data from the AFL website

function processReqChange () {

    // only if when "complete"
    if (xmlhttp.readyState == 4) {
        debugAlert("processReqChange: request is complete");
        // only if "OK"
        if (xmlhttp.status == 200) {
            processReply (xmlhttp, ladderURL);
         } 
        else {
            debugAlert("processReqChange: status != 200");
            // check if we have run out of data sources to check
            if (ladderURL == sourceURLs.length - 1) {
                debugAlert("processReqChange: no more data sources to try!");
                document.getElementById("progress").style.visibility = "hidden";
                 document.getElementById("error").style.visibility = "visible";
                xmlhttp.abort();
                xmlhttp = false;
            }
            else {
                debugAlert("processReqChange: trying next data source");
                // move on to next data source
                // try the next data source
                xmlhttp.abort();    
                
                xmlhttp = false;         
                processingRequest1 = false;
                retryFetch();
            }
        }
        processingRequest1 = false;
    }
    if (xmlhttp.readyState == 0) {
        // loading
        debugAlert("processReqChange: request is not initialised");

    }
    if (xmlhttp.readyState == 1) {
        // loading
        debugAlert("processReqChange: request has been set up");

    }
    if (xmlhttp.readyState == 2) {
        // loading
        debugAlert("processReqChange: request has been sent");

    }
    if (xmlhttp.readyState == 3) {
        // loading
        debugAlert("processReqChange: request is in process");
    }
}

//----------------------
// This function handles the fetching of the version data from expat.dyndns.org
// ignores errors when communicating back to home base.

function processReqChangeUpdate () {
	
    // only if when "complete"
    if (xmlhttp2.readyState == 4) {
        // only if "OK"
        if (xmlhttp2.status == 200) {
            processReplyUpdate (xmlhttp2);
		 } 
        else {
            xmlhttp2.abort();
            xmlhttp2 = false;
        }
    }
}

//---------------------------------------------------------------------
// process reply from expat.dydnsn.org to get latest version info

function processReplyUpdate (xmlResponse) {
    try {   
        // extract the ladder from the reply data
        var response_doc = document.implementation.createHTMLDocument("");
        response_doc.open();
        response_doc.write(xmlResponse.responseText); 
		
        var replydata = response_doc.getElementById("versionNumber");
		if (replydata.innerHTML != null) {
			if (replydata.innerHTML > version) {
				newVersionAvailable = 1;
			}
			else {
                // hide the update image
				newVersionAvailable = 0;
			}
		}
		else {
			newVersionAvailable = 0;
		}

		if (newVersionAvailable == 0) {
			debugAlert("processReplyUpdate: NO update available");	
            // hide the update image
            document.getElementById("updateImg").style.visibility = "hidden";
		}
        else {
            debugAlert("processReplyUpdate: update available");
            // set the update image visible
            document.getElementById("updateImg").style.visibility = "visible";
        }

		// Clean up some data
        replydata = false;
	}
	catch (ex) {
		// This is the exception handler for the above processing
		// if any of the requests fail we end up here
		debugAlert("processReplyUpdate: Error with update data!");
	}
    
   // free the downloaded oject & data
   processingRequest2 = false;
   response_doc = false;
   xmlResponse = false;
   xmlhttp2 = false;
}

//----------------------
// function to handle debugging modes

function debugAlert(string) {
	if (debugAlerts == true) {
		alert(string);
	}
}

//----------------------
// Process the data returned from sucessfully fetching the ladder data from the various data sources

function processReply (xmlResponse, url) {
    // name, P, W, D, L, F, A, %, Total
    var team = "";
    var played = 0;
    var wins = 0;
    var losses = 0;
    var draws = 0;
    var points = 0;
	var website = 0;
    var ladderIndex = 0;
	var finished = 0;
	var history = 0;
    var teamCounter = 0;
    var retry = false;

    try {
        if (url == aflURL) {
            // the data came from the AFL website
             debugAlert("processReply: processing data from " + sourceURLs[aflURL][2]);
            // extract the ladder from the reply data using XML ninja skills

            var xmlData = xmlResponse.responseXML;
            var teams = xmlData.getElementsByTagName("team");

            // make sure all the teams are there in the reply
            if (teams.length != teamData.length) {
                debugAlert("processReply: teams.length = " + teams.length);
                throw new Error ("processReply: Can't find all the team tags");
            }

            // Now we can build the ladderArray
            
            for (var k = 0; k < ladderArray.length; k++) {

                // find the short name associated with the teamid
                for (var n = 0; n < teamData.length; n++) {
                    if (teamData[n][0] == teams[k].getAttribute("teamid")) {
                        ladderArray[k][0] = teamData[n][1];
                        break;
                    }
                }
                                            
                var details = teams[k].getElementsByTagName("details");
                if (details.length == 0) {
                    throw new Error ("processReply: Can't find team details tag");
                }
                
                // debug statements to give realistic-like ladder data
                // note that there's a bug in the Widget engine that makes
                // Math.random repeat
                if (debugData == true) {
                    p = Math.ceil(Math.random() * 23);
                    w = Math.ceil(Math.random() * 22);
                    d = Math.ceil(Math.random() * 3);
                    l = Math.ceil(Math.random() * 23 + 3);
                    points = Math.ceil(Math.random()  * 88);
                    f = Math.ceil(Math.random() * 1000 + 200);
                    a = Math.ceil(Math.random() * 1000 + 200)
                    if (a > 0) {
                        percentage = (f/a) * 100;
                    }
                    else {
                        percentage = 0;
                    }
                    details[0].setAttribute("points", points);
                    details[0].setAttribute("f", f);
                    details[0].setAttribute("a", a);
                    details[0].setAttribute("p", p);
                    details[0].setAttribute("w", w);
                    details[0].setAttribute("l", l);
                    details[0].setAttribute("d", d);
                }
                // end of debug routine
                //-----------------	
                            
                ladderArray[k][1] = details[0].getAttribute("p");
                ladderArray[k][2] = details[0].getAttribute("w");
                ladderArray[k][3] = details[0].getAttribute("l");
                ladderArray[k][4] = details[0].getAttribute("d");
                ladderArray[k][5] = details[0].getAttribute("f");
                ladderArray[k][6] = details[0].getAttribute("a");
                    
                if (details[0].getAttribute("a") > 0) {
                    ladderArray[k][7] = round_decimals((details[0].getAttribute("f") / details[0].getAttribute("a") * 100), 2);
                }
                else {
                    ladderArray[k][7] = "0.00";
                }
                ladderArray[k][8] = details[0].getAttribute("points");
                ladderArray[k][9] = teams[k].getAttribute("website");
                        
                // get match data for past numberOfPastGames games
                var matches = teams[k].getElementsByTagName("m");
                    
                if (numberOfPastGames > matches.length) {
                    debugAlert("processReply: numberOfPastGames " + numberOfPastGames + " > length of data " + matches.length);
                    numberOfPastGames = matches.length;
                }
                        
                ladderArray[k][10] = "For:\t\t\t" + ladderArray[k][5] + "\n";
                ladderArray[k][10] = ladderArray[k][10] + "Against:\t\t" + ladderArray[k][6] + "\n";
                ladderArray[k][10] = ladderArray[k][10] + "Percentage:\t" + ladderArray[k][7] + " %";
                        
                // make sure the matches array has something in it!
                if (matches.length > 0) {
                    ladderArray[k][10] = ladderArray[k][10] + "\n\n";
                    
                    numberOfPastGames = matches.length;
                    ladderArray[k][10] = ladderArray[k][10] + "Last "
                    
                    if (numberOfPastGames > 6) {
                        numberOfPastGames = 6;
                    }        
                    if (numberOfPastGames == 1) {
                        ladderArray[k][10] = ladderArray[k][10] + "Result:\n";
                    }
                    else {
                        ladderArray[k][10] = ladderArray[k][10] + numberOfPastGames + " Results:\n";
                    }
                    
                    // format past game data                                                                                                           
                    for (var j = 0; j < numberOfPastGames; j++) {
                        // get the information we need from the XML file
                        var res = matches[j].getAttribute("res");
                        var score = matches[j].getAttribute("score");
                        var ven = matches[j].getAttribute("ven");
                        var length = 0;

                        // reformat some of the data
                        if (res == "Lose") {
                            ladderArray[k][10] = ladderArray[k][10] + "L\t";
                            length = length + 4;
                        }
                        if (res == "Win") {
                            ladderArray[k][10] = ladderArray[k][10] + "W\t";
                            length = length + 4;
                        }
                        if (res == "Draw") {
                            ladderArray[k][10] = ladderArray[k][10] + "D\t";
                            length = length + 4;
                        }
                        // Handle some mess ups that can occur in the data from the AFL
                        if (score != "") {
                            // pad out score to 15 characters to make formatting uniform
                            for (var e = score.length; e < 15; e++) {
                                score = score + " ";
                            }
                            ladderArray[k][10] = ladderArray[k][10] + score + "\t";
                        }
                        else {
                            ladderArray[k][10] = ladderArray[k][10] + "\t\t";
                        }
                            
                        // use shorter names for some venues, otherwise don't 
                        // change the venue string
                        for (var p = 0; p < groundNames.length; p++) {
                            if (groundNames[p][0] == ven) {
                                ven = groundNames[p][1];
                                break;
                            }
                        }						

                        // but change the venue to upper case
                        ladderArray[k][10] = ladderArray[k][10] + ven.toUpperCase();
                            
                        // don't add a new line after the last result line
                        if (j < (numberOfPastGames - 1)) {
                            ladderArray[k][10] = ladderArray[k][10] + "\n";
                        }
                    }
                }
                else {
                    debugAlert("processReply: No past result data found!");
                } 
                // make double sure that we have the right total of teams.
                teamCounter++;
            }

            debugAlert ("processReply: Passed team name check, teamCounter = " + teamCounter);
            
            ladderArray.sort(teamSortFn);
            debugAlert("processReply: Sorted team Order");
		}
        else if (url == sportalURL) {
            // data from sportal.com.au
            debugAlert("processReply: processing data from " + sourceURLs[sportalURL][2]);
            
            // extract the ladder from the reply data
            var response_doc = document.implementation.createHTMLDocument("");
            response_doc.open();
            response_doc.write(xmlResponse.responseText);
            
            var infoTable = response_doc.getElementsByTagName("table")[0];
            var rowList = infoTable.getElementsByTagName("tr");  
            debugAlert("processReply: rowList.length = " + rowList.length);
            if (rowList.length == 19) {
                for (var row = 0; row < rowList.length; row++) {
                    // skip rows with no data
                    if ((row != 0)) { 
                        var cells = rowList[row].getElementsByTagName("td");
        
                        // name, adjust long names to fix formatting issues 
                        var longName = cleanCell(cells[1]);
                        var i = 0;
                        for (i = 0; i < teamData.length; i++) {
                            if (teamData[i][3 + sportalURL] == longName) {
                                ladderArray[ladderIndex][0] = teamData[i][1];
                                break;
                            }
                        }
                        if (debugData == false) {
                            // played
                            ladderArray[ladderIndex][1] = cleanCell(cells[2]);
                            // won
                            ladderArray[ladderIndex][2] = cleanCell(cells[3]);
                            // lost
                            ladderArray[ladderIndex][3] = cleanCell(cells[5]);
                            // drawn
                            ladderArray[ladderIndex][4] = cleanCell(cells[4]);
                            // for
                            ladderArray[ladderIndex][5] = cleanCell(cells[6]);
                            // against
                            ladderArray[ladderIndex][6] = cleanCell(cells[7]);     
                            // %
                            ladderArray[ladderIndex][7] = cleanCell(cells[8]);
                            // Points
                            ladderArray[ladderIndex][8] = cleanCell(cells[9]);
                        }
                        else {
                                                        // played
                            ladderArray[ladderIndex][1] = Math.ceil(Math.random() * 23);
                            // won
                            ladderArray[ladderIndex][2] = Math.ceil(Math.random() * 22);
                            // lost
                            ladderArray[ladderIndex][3] = Math.ceil(Math.random() * 23 + 3);
                            // drawn
                            ladderArray[ladderIndex][4] = Math.ceil(Math.random() * 3);
                            // for
                            ladderArray[ladderIndex][5] = Math.ceil(Math.random() * 1000 + 200);
                            // against
                            ladderArray[ladderIndex][6] = Math.ceil(Math.random() * 1000 + 200) 
                            // %
                            if (ladderArray[ladderIndex][6] > 0) {
                                ladderArray[ladderIndex][7] = round_decmials(((ladderArray[ladderIndex][5]/ladderArray[ladderIndex][6]) * 100), 2);
                            }
                            else {
                                pladderArray[ladderIndex][7] = 0;
                            }
                            // Points
                            ladderArray[ladderIndex][8] = Math.ceil(Math.random()  * 88);
                        }
                        // website
                        ladderArray[ladderIndex][9] = teamData[i][2];
                        // tooltips
                        ladderArray[ladderIndex][10] = "For:\t\t\t" + ladderArray[ladderIndex][5] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Against:\t\t" + ladderArray[ladderIndex][6] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Percentage:\t" + ladderArray[ladderIndex][7] + " %";
                        ladderIndex++;
                        teamCounter++;
                    }
                }
            }
            else {
                throw new Error ("processReply: Can't find the right number of table rows from " + sourceURLs[sportalURL][2]);
            }
        }
        else if (url == heraldsunURL) {
            // data from heraldsun.com.au
            debugAlert("processReply: processing data from " + sourceURLs[heraldsunURL][2]);
            
            // extract the ladder from the reply data
            var response_doc = document.implementation.createHTMLDocument("");
            response_doc.open();
            response_doc.write(xmlResponse.responseText);
            
            var infoTable = response_doc.getElementsByTagName("table")[0];
            var rowList = infoTable.getElementsByTagName("tr");  
            debugAlert("processReply: rowList.length = " + rowList.length);
            if (rowList.length == 20) {
                for (var row = 0; row < rowList.length; row++) {
                    if (row > 1) {
                        var head =  rowList[row].getElementsByTagName("th");
                        var cells = rowList[row].getElementsByTagName("td");
        
                        // name, adjust long names to fix formatting issues 
                        var longName = cleanCell(head[0]);
                        var i = 0;
                        for (i = 0; i < teamData.length; i++) {
                            if (teamData[i][3 + heraldsunURL] == longName) {
                                ladderArray[ladderIndex][0] = teamData[i][1];
                                break;
                            }
                        }
                        if (debugData == false) {
                            // played
                            ladderArray[ladderIndex][1] = cleanCell(cells[0]);
                            // won
                            ladderArray[ladderIndex][2] = cleanCell(cells[1]);
                            // lost
                            ladderArray[ladderIndex][3] = cleanCell(cells[2]);
                            // drawn
                            ladderArray[ladderIndex][4] = cleanCell(cells[3]);
                            // for
                            ladderArray[ladderIndex][5] = cleanCell(cells[4]);
                            // against
                            ladderArray[ladderIndex][6] = cleanCell(cells[5]);     
                            // %
                            ladderArray[ladderIndex][7] = cleanCell(cells[6]);
                            // Points
                            ladderArray[ladderIndex][8] = cleanCell(cells[7]);
                        }
                        else {
                            // played
                            ladderArray[ladderIndex][1] = Math.ceil(Math.random() * 23);
                            // won
                            ladderArray[ladderIndex][2] = Math.ceil(Math.random() * 22);
                            // lost
                            ladderArray[ladderIndex][3] = Math.ceil(Math.random() * 23 + 3);
                            // drawn
                            ladderArray[ladderIndex][4] = Math.ceil(Math.random() * 3);
                            // for
                            ladderArray[ladderIndex][5] = Math.ceil(Math.random() * 1000 + 200);
                            // against
                            ladderArray[ladderIndex][6] = Math.ceil(Math.random() * 1000 + 200) 
                            // %
                            if (ladderArray[ladderIndex][6] > 0) {
                                ladderArray[ladderIndex][7] = round_decmials(((ladderArray[ladderIndex][5]/ladderArray[ladderIndex][6]) * 100), 2);
                            }
                            else {
                                pladderArray[ladderIndex][7] = 0;
                            }
                            // Points
                            ladderArray[ladderIndex][8] = Math.ceil(Math.random()  * 88);
                        }
                        // website
                        ladderArray[ladderIndex][9] = teamData[i][2];
                        // tooltips
                        ladderArray[ladderIndex][10] = "For:\t\t\t" + ladderArray[ladderIndex][5] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Against:\t\t" + ladderArray[ladderIndex][6] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Percentage:\t" + ladderArray[ladderIndex][7] + " %";       
                        ladderIndex++;
                        teamCounter++;
                    }
                }
            }
            else {
                throw new Error ("processReply: Can't find the right number of table rows from " + sourceURLs[heraldsunURL][2]);
            }
        }
        else if (url == smhURL) {
            // data from smh.com.au
            debugAlert("processReply: processing data from " + sourceURLs[smhURL][2]);
            
            // extract the ladder from the reply data
            var response_doc = document.implementation.createHTMLDocument("");
            response_doc.open();
            response_doc.write(xmlResponse.responseText);
            
            var infoTable = response_doc.getElementsByTagName("table")[0];
            var rowList = infoTable.getElementsByTagName("tr");  
            debugAlert("processReply: rowList.length = " + rowList.length);
            if (rowList.length == 20) {
                for (var row = 0; row < rowList.length; row++) {
                    debugAlert("processReply: smhURL row = " + row);
                    // skip rows with no data
                    if ((row != 0) && (row != 1)) {
                        var cells = rowList[row].getElementsByTagName("td");
        
                        // name, adjust long names to fix formatting issues
                        var longName = cleanCell(cells[0]);
                        var i = 0;
                        for (i = 0; i < teamData.length; i++) {
                            if (teamData[i][3 + smhURL] == longName) {
                                ladderArray[ladderIndex][0] = teamData[i][1];
                                break;
                            }
                        }
                        if (debugData == false) {
                            // played
                            ladderArray[ladderIndex][1] = cleanCell(cells[1]);
                            // won
                            ladderArray[ladderIndex][2] = cleanCell(cells[2]);
                            // lost
                            ladderArray[ladderIndex][3] = cleanCell(cells[3]);
                            // drawn
                            ladderArray[ladderIndex][4] = cleanCell(cells[4]);
                            // for
                            ladderArray[ladderIndex][5] = cleanCell(cells[5]);
                            // against
                            ladderArray[ladderIndex][6] = cleanCell(cells[6]);     
                            // %
                            ladderArray[ladderIndex][7] = cleanCell(cells[7]);
                            // Points
                            ladderArray[ladderIndex][8] = cleanCell(cells[8]);
                        }
                        else {
                                                        // played
                            ladderArray[ladderIndex][1] = Math.ceil(Math.random() * 23);
                            // won
                            ladderArray[ladderIndex][2] = Math.ceil(Math.random() * 22);
                            // lost
                            ladderArray[ladderIndex][3] = Math.ceil(Math.random() * 23 + 3);
                            // drawn
                            ladderArray[ladderIndex][4] = Math.ceil(Math.random() * 3);
                            // for
                            ladderArray[ladderIndex][5] = Math.ceil(Math.random() * 1000 + 200);
                            // against
                            ladderArray[ladderIndex][6] = Math.ceil(Math.random() * 1000 + 200) 
                            // %
                            if (ladderArray[ladderIndex][6] > 0) {
                                ladderArray[ladderIndex][7] = round_decmials(((ladderArray[ladderIndex][5]/ladderArray[ladderIndex][6]) * 100), 2);
                            }
                            else {
                                pladderArray[ladderIndex][7] = 0;
                            }
                            // Points
                            ladderArray[ladderIndex][8] = Math.ceil(Math.random()  * 88);
                        }
                        // website
                        ladderArray[ladderIndex][9] = teamData[i][2];
                        // tooltips
                        ladderArray[ladderIndex][10] = "For:\t\t\t" + ladderArray[ladderIndex][5] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Against:\t\t" + ladderArray[ladderIndex][6] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Percentage:\t" + ladderArray[ladderIndex][7] + " %";
                        ladderIndex++;
                        teamCounter++;
                    }
                }
            }
            else {
                throw new Error ("processReply: Can't find the right number of table rows from " + sourceURLs[heraldsunURL][2]);
            }
        }
        else if (url == foxsportsURL) {
            // data from foxsports
            debugAlert("processReply: processing data from " + sourceURLs[foxsportsURL][2]);
            
            // extract the ladder from the reply data
            var response_doc = document.implementation.createHTMLDocument("");
            response_doc.open();
            response_doc.write(xmlResponse.responseText);
            
            var infoTable = response_doc.getElementsByTagName("table")[0];
            var rowList = infoTable.getElementsByTagName("tr");  
            debugAlert("processReply: rowList.length = " + rowList.length);
            if (rowList.length == 19) {
                for (var row = 0; row < rowList.length; row++) {
                    debugAlert("processReply: foxsportsURL row = " + row);
                    // skip rows with no data
                    if (row != 0) {
                        var cells = rowList[row].getElementsByTagName("td");
        
                        // name, adjust long names to fix formatting issues
                        var longName = cleanCell(cells[1]);
                        var i = 0;
                        for (i = 0; i < teamData.length; i++) {
                            if (teamData[i][3 + foxsportsURL] == longName) {
                                ladderArray[ladderIndex][0] = teamData[i][1];
                                break;
                            }
                        }
                        if (debugData == false) {
                            // played
                            ladderArray[ladderIndex][1] = cleanCell(cells[2]);
                            // won
                            ladderArray[ladderIndex][2] = cleanCell(cells[3]);
                            // lost
                            ladderArray[ladderIndex][3] = cleanCell(cells[5]);
                            // drawn
                            ladderArray[ladderIndex][4] = cleanCell(cells[4]);
                            // for
                            ladderArray[ladderIndex][5] = 0; //cleanCell(cells[5]);
                            // against
                            ladderArray[ladderIndex][6] = 0; //cleanCell(cells[6]);     
                            // %
                            ladderArray[ladderIndex][7] = cleanCell(cells[6]);
                            // Points
                            ladderArray[ladderIndex][8] = cleanCell(cells[7]);
                        }
                        else {
                                                        // played
                            ladderArray[ladderIndex][1] = Math.ceil(Math.random() * 23);
                            // won
                            ladderArray[ladderIndex][2] = Math.ceil(Math.random() * 22);
                            // lost
                            ladderArray[ladderIndex][3] = Math.ceil(Math.random() * 23 + 3);
                            // drawn
                            ladderArray[ladderIndex][4] = Math.ceil(Math.random() * 3);
                            // for
                            ladderArray[ladderIndex][5] = Math.ceil(Math.random() * 1000 + 200);
                            // against
                            ladderArray[ladderIndex][6] = Math.ceil(Math.random() * 1000 + 200) 
                            // %
                            if (ladderArray[ladderIndex][6] > 0) {
                                ladderArray[ladderIndex][7] = round_decmials(((ladderArray[ladderIndex][5]/ladderArray[ladderIndex][6]) * 100), 2);
                            }
                            else {
                                pladderArray[ladderIndex][7] = 0;
                            }
                            // Points
                            ladderArray[ladderIndex][8] = Math.ceil(Math.random()  * 88);
                        }
                        // website
                        ladderArray[ladderIndex][9] = teamData[i][2];
                        // tooltips
                        ladderArray[ladderIndex][10] = "For:\t\t\t" + ladderArray[ladderIndex][5] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Against:\t\t" + ladderArray[ladderIndex][6] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Percentage:\t" + ladderArray[ladderIndex][7] + " %";
                        ladderIndex++;
                        teamCounter++;
                    }
                }
            }
            else {
                throw new Error ("processReply: Can't find the right number of table rows from " + sourceURLs[foxsportsURL][2]);
            }
        }
        else if (url == yahoo7URL) {
            // data from yahoo7
            debugAlert("processReply: processing data from " + sourceURLs[yahoo7URL][2]);
            
            // extract the ladder from the reply data
            var response_doc = document.implementation.createHTMLDocument("");
            response_doc.open();
            response_doc.write(xmlResponse.responseText);
            
            var infoTable = response_doc.getElementsByTagName("table")[0];
            var rowList = infoTable.getElementsByTagName("tr");  
            debugAlert("processReply: rowList.length = " + rowList.length);
            if (rowList.length == 20) {
                for (var row = 0; row < rowList.length; row++) {
                    debugAlert("processReply: yahoo7URL row = " + row);
                    // skip rows with no data
                    if ((row != 0) && (row != 9)) {
                        var cells = rowList[row].getElementsByTagName("td");
        
                        // name, adjust long names to fix formatting issues
                        var longName = cleanCell(cells[1]);
                        var i = 0;
                        for (i = 0; i < teamData.length; i++) {
                            if (teamData[i][3 + yahoo7URL] == longName) {
                                ladderArray[ladderIndex][0] = teamData[i][1];
                                break;
                            }
                        }
                        if (debugData == false) {
                            // played
                            ladderArray[ladderIndex][1] = cleanCell(cells[2]);
                            // won
                            ladderArray[ladderIndex][2] = cleanCell(cells[3]);
                            // lost
                            ladderArray[ladderIndex][3] = cleanCell(cells[5]);
                            // drawn
                            ladderArray[ladderIndex][4] = cleanCell(cells[4]);
                            // for
                            ladderArray[ladderIndex][5] = cleanCell(cells[6]);
                            // against
                            ladderArray[ladderIndex][6] = cleanCell(cells[7]);     
                            // %
                            if (cleanCell(cells[8]) == undefined) {
                                ladderArray[ladderIndex][7] = 0
                            }
                            else {
                                ladderArray[ladderIndex][7] = cleanCell(cells[8]);
                            }
                            // Points
                            ladderArray[ladderIndex][8] = cleanCell(cells[9]);
                        }
                        else {
                                                        // played
                            ladderArray[ladderIndex][1] = Math.ceil(Math.random() * 23);
                            // won
                            ladderArray[ladderIndex][2] = Math.ceil(Math.random() * 22);
                            // lost
                            ladderArray[ladderIndex][3] = Math.ceil(Math.random() * 23 + 3);
                            // drawn
                            ladderArray[ladderIndex][4] = Math.ceil(Math.random() * 3);
                            // for
                            ladderArray[ladderIndex][5] = Math.ceil(Math.random() * 1000 + 200);
                            // against
                            ladderArray[ladderIndex][6] = Math.ceil(Math.random() * 1000 + 200) 
                            // %
                            if (ladderArray[ladderIndex][6] > 0) {
                                ladderArray[ladderIndex][7] = round_decmials(((ladderArray[ladderIndex][5]/ladderArray[ladderIndex][6]) * 100), 2);
                            }
                            else {
                                pladderArray[ladderIndex][7] = 0;
                            }
                            // Points
                            ladderArray[ladderIndex][8] = Math.ceil(Math.random()  * 88);
                        }
                        // website
                        ladderArray[ladderIndex][9] = teamData[i][2];
                        // tooltips
                        ladderArray[ladderIndex][10] = "For:\t\t\t" + ladderArray[ladderIndex][5] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Against:\t\t" + ladderArray[ladderIndex][6] + "\n";
                        ladderArray[ladderIndex][10] = ladderArray[ladderIndex][10] + "Percentage:\t" + ladderArray[ladderIndex][7] + " %";
                        ladderIndex++;
                        teamCounter++;
                    }
                }
            }
            else {
                throw new Error ("processReply: Can't find the right number of table rows from " + sourceURLs[yahoo7URL][2]);
            }
        }        
        else if (url == finalsirenURL) {
            // data from finalsiren.com
            debugAlert("processReply: processing data from " + sourceURLs[finalsirenURL][2]);
            
            // extract the ladder from the reply data
            var response_doc = document.implementation.createHTMLDocument("");
            response_doc.open();
            response_doc.write(xmlResponse.responseText);
            
            var infoTable = response_doc.getElementsByTagName("table")[1];
            var rowList = infoTable.getElementsByTagName("tr");  
            debugAlert("processReply: rowList.length = " + rowList.length);
            if (rowList.length == 20) {
                for (var row = 0; row < rowList.length; row++) {
                    // skip rows with no data
                    if (row > 1) {
                        var cells = rowList[row].getElementsByTagName("td");
        
                        // name, adjust long names to fix formatting issues 
                        var longName = cleanCell(cells[1]);
                        var i = 0;
                        for (i = 0; i < teamData.length; i++) {
                            if (teamData[i][3+ finalsirenURL] == longName) {
                                ladderArray[ladderIndex][0] = teamData[i][1];
                                break;
                            }
                        }

                        if (debugData == false) {
                            // won
                            ladderArray[ladderIndex][2] = cleanCell(cells[3]);
                            // lost
                            ladderArray[ladderIndex][3] = cleanCell(cells[5]);
                            // drawn
                            ladderArray[ladderIndex][4] = cleanCell(cells[4]);
                            // played
                            ladderArray[ladderIndex][1] = cleanCell(cells[2]);  //parseFloat(ladderArray[ladderIndex][2]) + parseFloat(ladderArray[ladderIndex][3]) + parseFloat(ladderArray[ladderIndex][4]);
                            // for
                            ladderArray[ladderIndex][5] = cleanCell(cells[6]);
                            // against
                            ladderArray[ladderIndex][6] = cleanCell(cells[7]);
                            // try to workout which is the points column by checking if the points column contain
                            // a string or a number
                            debugAlert("processReply: finalSiren row = " + row);
                            if (IsNumeric(cleanCell(cells[17]))) {
                                // if it is the points column if it contains a number.
                                // %
                                ladderArray[ladderIndex][7] = cleanCell(cells[18]);
                                // Points
                                ladderArray[ladderIndex][8] = cleanCell(cells[17]);
                            }
                            else {
                                // if the guess for the points column contains a string, then it's likely to be the "Chg"
                                // column which only appears after round 1
                                // %
                                ladderArray[ladderIndex][7] = cleanCell(cells[19]);
                                // Points
                                ladderArray[ladderIndex][8] = cleanCell(cells[18]);
                            }
                        }
                        else {
                                                        // played
                            ladderArray[ladderIndex][1] = Math.ceil(Math.random() * 23);
                            // won
                            ladderArray[ladderIndex][2] = Math.ceil(Math.random() * 22);
                            // lost
                            ladderArray[ladderIndex][3] = Math.ceil(Math.random() * 23 + 3);
                            // drawn
                            ladderArray[ladderIndex][4] = Math.ceil(Math.random() * 3);
                            // for
                            ladderArray[ladderIndex][5] = Math.ceil(Math.random() * 1000 + 200);
                            // against
                            ladderArray[ladderIndex][6] = Math.ceil(Math.random() * 1000 + 200) 
                            // %
                            if (ladderArray[ladderIndex][6] > 0) {
                                ladderArray[ladderIndex][7] = round_decimals(((ladderArray[ladderIndex][5]/ladderArray[ladderIndex][6]) * 100), 2);
                            }
                            else {
                                ladderArray[ladderIndex][7] = 0;
                            }
                            // Points
                            ladderArray[ladderIndex][8] = Math.ceil(Math.random()  * 88);
                        }
                        // website
                        ladderArray[ladderIndex][9] = teamData[i][2];
                        
                        ladderIndex++;
                        teamCounter++;
                    }
                }
            }
            else {
                throw new Error ("processReply: Can't find the right number of table rows from " + sourceURLs[finalsirenURL][2]);
            }
        }
        else if (url == finalsirenXML) {
            // the data came from the finalsiren.com website using XML
             debugAlert("processReply: processing data from " + sourceURLs[finalsirenXML][2]);
            // extract the ladder from the reply data using XML ninja skills

            var xmlData = xmlResponse.responseXML;
            var teams = xmlData.getElementsByTagName("team");

            // make sure all the teams are there in the reply
            if (teams.length != teamData.length) {
                debugAlert("processReply: teams.length = " + teams.length);
                throw new Error ("processReply: Can't find all the team tags");
            }

            // Now we can start to build the ladderArray
            for (var k = 0; k < ladderArray.length; k++) {

                // find the short name associated with the teamname in the XML file
                for (var n = 0; n < teamData.length; n++) {
                    if (teamData[n][finalsirenXML + 3] == teams[k].getAttribute("teamname")) {
                        ladderArray[k][0] = teamData[n][1];   // copy short name
                        ladderArray[k][9] = teamData[n][2];    // copy website URL
                        break;
                    }
                }
                
                // debug statements to give realistic-like ladder data
                // note that there's a bug in the Widget engine that makes
                // Math.random repeat
                if (debugData == true) {
                    p = Math.ceil(Math.random() * 23);
                    w = Math.ceil(Math.random() * 22);
                    d = Math.ceil(Math.random() * 3);
                    l = Math.ceil(Math.random() * 23 + 3);
                    points = Math.ceil(Math.random()  * 88);
                    f = Math.ceil(Math.random() * 1000 + 200);
                    a = Math.ceil(Math.random() * 1000 + 200)
                    if (a > 0) {
                        percentage = (f/a) * 100;
                    }
                    else {
                        percentage = 0;
                    }
                    details[0].setAttribute("points", points);
                    details[0].setAttribute("f", f);
                    details[0].setAttribute("a", a);
                    details[0].setAttribute("p", p);
                    details[0].setAttribute("w", w);
                    details[0].setAttribute("l", l);
                    details[0].setAttribute("d", d);
                }
                // end of debug routine
                //-----------------	
                            
                ladderArray[k][2] = parseInt(teams[k].getAttribute("wins"));
                ladderArray[k][3] = parseInt(teams[k].getAttribute("losses"));
                ladderArray[k][4] = parseInt(teams[k].getAttribute("draws"));
                
                ladderArray[k][1] = ladderArray[k][2] + ladderArray[k][3];
                
                ladderArray[k][5] = parseInt(teams[k].getAttribute("pointsfor"));
                ladderArray[k][6] = parseInt(teams[k].getAttribute("pointsagainst"));
                    
                if (parseInt(teams[0].getAttribute("pointsagainst")) > 0) {
                    ladderArray[k][7] = round_decimals((parseInt(teams[k].getAttribute("pointsfor")) / parseInt(teams[k].getAttribute("pointsagainst")) * 100), 2);
                }
                else {
                    ladderArray[k][7] = "0.00";
                }
                ladderArray[k][8] = parseInt(teams[k].getAttribute("points"));
                       
                ladderArray[k][10] = "For:\t\t\t" + ladderArray[k][5] + "\n";
                ladderArray[k][10] = ladderArray[k][10] + "Against:\t\t" + ladderArray[k][6] + "\n";
                ladderArray[k][10] = ladderArray[k][10] + "Percentage:\t" + ladderArray[k][7] + " %";

                // make double sure that we have the right total of teams.
                teamCounter++;
            }

            debugAlert ("processReply: Passed team name check, teamCounter = " + teamCounter);
            
            ladderArray.sort(teamSortFn);
            debugAlert("processReply: Sorted team Order");
		}
        else {
            throw new Error ("processReply: unable to process reply data from url = " + url);
        }
        
        // Check the ladderArray structure for all the team names and no "undefined" values.
        teamCounter = 0;
        for (var i = 0; i < teamData.length; i++) {
            for (j = 0; j < ladderArray.length; j++) {
                if (ladderArray[j][0] == teamData[i][1]) {
                    debugAlert("processReply: data validation found team = " + ladderArray[j][0]);
                    for (var k = 1; k < ladderArray[j].length; k++) {
                        if (ladderArray[j][k] == undefined) {
                            debugAlert("processReply: data validation found undefined value, i = " + i + " j = " + j + " k = " + k);
                            ladderArray[j][k] = 0;
                        }
                    }
                    teamCounter++;
                    break;
                }
            }
        }
        
		// Found all the teamsIds in the data we got back so we're
		// pretty safe to assume that we've fetched the table data ok
		// so we can start to load up the table
		if (teamCounter == teamData.length) {
            debugAlert("processReply: building and displaying ladder");

			// update the widget with the new data
            for (var i = 0; i < ladderArray.length; i++) {
                teamArrayLadder._rowData[i] = ladderArray[i][0];
                playedArrayLadder._rowData[i] = ladderArray[i][1];
                wonArrayLadder._rowData[i] = ladderArray[i][2];
                lostArrayLadder._rowData[i] = ladderArray[i][3];
                drawnArrayLadder._rowData[i] = ladderArray[i][4];
                pointsArrayLadder._rowData[i] = ladderArray[i][8];
                percentArrayLadder._rowData[i] = ladderArray[i][7];
            }

            // check favourite team highlight
            highlightFavourite();
                                    
            // flag the ladder as loaded
			ladderLoaded = true;
            
            document.getElementById("teamList").object.reloadData();
			document.getElementById("playedList").object.reloadData();
            document.getElementById("wonList").object.reloadData();
            document.getElementById("lostList").object.reloadData();
            document.getElementById("drawnList").object.reloadData();
            document.getElementById("pointsList").object.reloadData();
            document.getElementById("percentList").object.reloadData();
            
			// show the table & remove the default txt & logo
			document.getElementById("progress").style.visibility = "hidden";
            complete = true;
		}
		else  {
			// remove the table & show default text & logo
            retry = true;
		}
		
		// Clean up some data
        tabledata = false;
	}    
	catch (ex) {
		// This is the exception handler for the above processing
		// if any of the requests fail we end up here
		ladderLoaded = false;
		debugAlert("processReply: Error with data!");
        debugAlert("processReply: URL = " + url);
        debugAlert("processReply: sourceURLs.length = " + sourceURLs.length);
        
        // check if we can check another data source
        
        if (ladderURL < (sourceURLs.length - 1)) {
            // no more sources to check
            retry = true;
        }
        else {
            retry = false;
        }
    }
    
   // free the downloaded oject & data
   response_doc = false;
   xmlResponse = false;
   xmlhttp = false;
   processingRequest1 = false;
   if (complete != true) {
        if (retry == true) {
            debugAlert("processReply: trying next data source");
            retryFetch();
        }
        else {
            debugAlert("processReply: No more data sources to try!");
            document.getElementById("progress").style.visibility = "hidden";
        }
    }
    else {
        debugAlert("processReply: finished fetching & displaying ladder data");
        document.getElementById("progress").style.visibility = "hidden";
        complete = false;
    }
}

//--------------------
// a function to sort a 2-D array on 3 variables
// points, then percentage, then team ref name

function teamSortFn (a,b) {
	// sort by points
	if (parseFloat(a[8]) > parseFloat(b[8])) return -1;
	if (Number(a[8]) < parseFloat(b[8])) return 1;
	if (parseFloat(a[8]) == parseFloat(b[8])) {
		// then by percentage
		if (parseFloat(a[7]) > parseFloat(b[7])) return -1;
		if (parseFloat(a[7]) < parseFloat(b[7])) return 1;
		if (parseFloat(a[7]) == parseFloat(b[7])) {
			// and last of all by team name
			if (a[0] > b[0]) return 1;
			if (a[0] < b[0]) return -1;
		}
	}
	return 0;
}

//--------------------

function round_decimals(original_number, decimals) {
    var result1 = original_number * Math.pow(10, decimals)
    var result2 = Math.round(result1)
    var result3 = result2 / Math.pow(10, decimals)
    return pad_with_zeros(result3, decimals)
}

//--------------------
function pad_with_zeros(rounded_value, decimal_places) {

    // Convert the number to a string
    var value_string = rounded_value.toString()
    
    // Locate the decimal point
    var decimal_location = value_string.indexOf(".")

    // Is there a decimal point?
    if (decimal_location == -1) {
        
        // If no, then all decimal places will be padded with 0s
        decimal_part_length = 0
        
        // If decimal_places is greater than zero, tack on a decimal point
        value_string += decimal_places > 0 ? "." : ""
    }
    else {

        // If yes, then only the extra decimal places will be padded with 0s
        decimal_part_length = value_string.length - decimal_location - 1
    }
    
    // Calculate the number of decimal places that need to be padded with 0s
    var pad_total = decimal_places - decimal_part_length
    
    if (pad_total > 0) {
        
        // Pad the string with 0s
        for (var counter = 1; counter <= pad_total; counter++) 
            value_string += "0"
        }
    return value_string
}

//------------------------
// if one data source fails then try the next until 
// we've tried them all
function retryFetch () {
    // find the next enabled URL
    do {
        ladderURL++;
    }
    while ((ladderURL < (sourceURLs.length - 1)) && (sourceURLs[ladderURL][0] != 1));

    // check that ladderURL is a valid index into sourceURLs
    if ((ladderURL < sourceURLs.length) && (sourceURLs[ladderURL][0] == 1)) {
        debugAlert("retryFetch: trying next URL");
        getLadderData(ladderURL);
    }
    else {
        debugAlert("retryFetch: No more URLs to try!");
        ladderURL = 0;
        document.getElementById("progress").style.visibility = "hidden";
        document.getElementById("error").style.visibility = "visible";
    }
}

//------------------------
// launch a browser with home URL
function openInBrowser () {
    widget.openURL(homeURL);
}

//-----------------------
// highlights the favourite team if selected
function highlightFavourite () {
    // debugging info
    if ((favourite > 0) && (favourite <= (ladderArray.length + 1))) {
        debugAlert ("highlightFavourite: Favourite Team = " + teamData[favourite - 1][1]);
    }
    else {
        debugAlert ("highlightFavourite: Favourite Team not defined");
    }
    
    for (var i = 0; i < ladderArray.length; i++) {
        // generate the ID for each table highlight position entity
        var highlightID = String("highlightPos");
        var offset = i + 1;
        highlightID = String(highlightID + offset);
                
        debugAlert("highlightFavourite: highlightID = " + highlightID);
        
        // check if it is the favourite and set the visibility of the highlight
        if ((favourite == 0) || (favourite > (ladderArray.length + 1))) {
            document.getElementById(highlightID).style.visibility = "hidden";
            favourite = 0;
        }
        else if (ladderArray[i][0] != teamData[favourite - 1][1]) {       
            document.getElementById(highlightID).style.visibility = "hidden";
        }
        else {
            document.getElementById(highlightID).style.visibility = "visible";
        }
    }
}

//---------------------------------------------------------------------
// this function handles the callback for selecting/deselecting the 
// updates checkbox

function toggleUpdates () {
	if (updates == true) {
		updates = false;
	}
	else {
		updates = true;
	}
	debugAlert("toggleUpdates: " + updates);
}

//---------------------------------------------------------------------
// this function checks a string to see if it is numeric

function IsNumeric(sText) {
   var ValidChars = "0123456789.";
   var IsNumber=true;
   var Char;

   if (sText == null) {
        return false;
   }
   for (i = 0; ((i < sText.length) && (IsNumber == true)); i++) 
      {
      Char = sText.charAt(i);
      if (ValidChars.indexOf(Char) == -1) 
         {
         IsNumber = false;
         }
      }
   return IsNumber;
}
